/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using MPLAB(c) Code Configurator

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  MPLAB(c) Code Configurator - 3.15.0
        Device            :  PIC16F1827
        Driver Version    :  2.00
    The generated drivers are tested against the following:
        Compiler          :  XC8 1.35
        MPLAB             :  MPLAB X 3.20
 */

/*
    (c) 2016 Microchip Technology Inc. and its subsidiaries. You may use this
    software and any derivatives exclusively with Microchip products.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION
    WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.

    MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
    TERMS.
 */

#include "mcc_generated_files/mcc.h"

#define MAX_SHOW_TIME 120
#define MAX_FALL_TIME 60
#define LEVEL_SHOW_TIME 20
#define LED_ON_PERIOD 255

typedef struct _CHANNEL {
    unsigned char level;
    unsigned char level_timeout;
    unsigned char max;
    unsigned char max_timeout;
    unsigned short leds;
    unsigned short treshold;
    adc_channel_t adc_channel;
} CHANNEL;

CHANNEL left;
CHANNEL right;

enum {
    lo_l,
    hi_l,
    lo_r,
    hi_r,
} phase;

unsigned char led_on_timer;

unsigned short tresholds[] = {3, 6, 10, 16, 28, 48, 81, 138, 235, 400, 420, 450};

unsigned short LevelToBits(unsigned char level) {
    unsigned short result = 0;
    unsigned short flag = 1;
    unsigned char i;
    for (i = 0; i < level; i++) {
        result |= flag;
        flag <<= 1;
    }
    return result;
}

unsigned short LevelToBit(unsigned char level) {
    unsigned short result;
    if (level > 0) {
        result = 1 << --level;
    } else {
        result = 0;
    }
    return result;
}

unsigned char GetLevel(adc_result_t value) {
    unsigned char result;
    for (result = 0; result < 12; result++) {
        if (value < tresholds[result])
            break;
    }
    return result;
}

void LedOff() {
    LED_L_LO_SetHigh();
    LED_L_HI_SetHigh();
    LED_R_LO_SetHigh();
    LED_R_HI_SetHigh();
    LED1_SetLow();
    LED2_SetLow();
    LED3_SetLow();
    LED4_SetLow();
    LED5_SetLow();
    LED6_SetLow();
}

void SetLowLeds(unsigned short level) {
    if (level & 0x0001) LED1_SetHigh();
    if (level & 0x0002) LED2_SetHigh();
    if (level & 0x0004) LED3_SetHigh();
    if (level & 0x0008) LED4_SetHigh();
    if (level & 0x0010) LED5_SetHigh();
    if (level & 0x0020) LED6_SetHigh();
}

void SetHighLeds(unsigned short level) {
    if (level & 0x0040) LED1_SetHigh();
    if (level & 0x0080) LED2_SetHigh();
    if (level & 0x0100) LED3_SetHigh();
    if (level & 0x0200) LED4_SetHigh();
    if (level & 0x0400) LED5_SetHigh();
    if (level & 0x0800) LED6_SetHigh();
}

void ReadChannel(CHANNEL* channel) {
    adc_result_t value = ADC_GetConversion(channel->adc_channel);
    if (value > channel->treshold) {
        value -= channel->treshold;

        unsigned char current_level = GetLevel(value);

        if (current_level >= channel->level) {
            channel->level = current_level;
            channel->level_timeout = LEVEL_SHOW_TIME;
        }

        if (current_level >= channel->max) {
            channel->max = current_level;
            channel->max_timeout = MAX_SHOW_TIME;
        }

        channel->leds = LevelToBits(channel->level);
        channel->leds |= LevelToBit(channel->max);
    } else {
        channel->leds = LevelToBit(channel->max);
    }
}

void DecrementMax(CHANNEL* channel) {

    if (channel->max_timeout != 0) {
        channel->max_timeout--;
        return;
    }

    if (channel->max != 0) {
        channel->max--;
        channel->max_timeout = MAX_FALL_TIME;
    }
}

void DecrementLevel(CHANNEL* channel) {

    if (channel->level_timeout != 0) {
        channel->level_timeout--;
        return;
    }

    if (channel->level != 0) {
        channel->level--;
        channel->level_timeout = LEVEL_SHOW_TIME;
    }
}

void Run(void) {

    DecrementLevel(&left);
    DecrementMax(&left);

    DecrementLevel(&right);
    DecrementMax(&right);

    ReadChannel(&left);
    ReadChannel(&right);

    switch (phase) {
        case lo_l:
            LedOff();
            SetLowLeds(left.leds);
            LED_L_LO_SetLow();
            phase = hi_l;
            break;
        case hi_l:
            LedOff();
            SetHighLeds(left.leds);
            LED_L_HI_SetLow();
            phase = lo_r;
            break;
        case lo_r:
            LedOff();
            SetLowLeds(right.leds);
            LED_R_LO_SetLow();
            phase = hi_r;
            break;
        case hi_r:
            LedOff();
            SetHighLeds(right.leds);
            LED_R_HI_SetLow();
            phase = lo_l;
            break;
        default:
            phase = lo_l;
    }

    led_on_timer = LED_ON_PERIOD;
    while (led_on_timer)
        led_on_timer--;
    LedOff();
}

void main(void) {
    // initialize the device
    SYSTEM_Initialize();

    left.adc_channel = channel_l;
    right.adc_channel = channel_r;
    
    left.treshold = ADC_GetConversion(channel_l);
    right.treshold = ADC_GetConversion(channel_r);

    TMR0_SetInterruptHandler(Run);

    // When using interrupts, you need to set the Global and Peripheral Interrupt Enable bits
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();

    while (1) {
        CLRWDT();
    }
}
/**
 End of File
 */